package demo.app.interfaces;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.headers.Header;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@RestController
@RequestMapping("/users")
public class DemoRestController {

    static final String DEMO_HEADER = "X-Demo-Header";
    static final String DEMO_HEADER_VALUE = "this_is_a_demo";

    @GetMapping(path = "/{id}", produces = APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.OK)
    @Operation(
            summary = "Get a user by id",
            responses = {
                    @ApiResponse(
                            responseCode = "200",
                            headers = {
                                    @Header(name = "X-Demo-Header",
                                            description = "Header used for demo purposes",
                                            schema = @Schema(implementation = String.class, accessMode = Schema.AccessMode.READ_ONLY))
                            },
                            description = "The specified user was found and is returned."),
                    @ApiResponse(
                            responseCode = "404",
                            description = "The specified user was not found.")})
    ResponseEntity<UserDto> getUser(@PathVariable("id") String id) {
        return ResponseEntity.ok()
                .header(DEMO_HEADER, DEMO_HEADER_VALUE)
                .body(new UserDto(id));
    }

    record UserDto(String id){}
}
