package demo.app.interfaces;

import demo.app.config.SwaggerConfig;
import demo.app.IntegrationTest;
import com.jayway.jsonpath.JsonPath;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import static org.assertj.core.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@IntegrationTest
public class ApiDocsTest {

    private static final String OPEN_API_SPEC_RESOURCE = "/v3/api-docs";

    @Autowired
    private MockMvc mockMvc;

    @Test
    void usersResourceAvailable_getApiDocs_apiDocsAreSuccessfullyReturned() throws Exception {

        String openApiSpecification = mockMvc.perform(get(OPEN_API_SPEC_RESOURCE)
                        .contentType(MediaType.APPLICATION_JSON)
                        .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andReturn()
                .getResponse()
                .getContentAsString();

        assertThat(openApiSpecification).isNotNull();

        var title = JsonPath.read(openApiSpecification, "$.info.title");
        assertThat(title).isEqualTo(SwaggerConfig.TITLE);

        var description = JsonPath.read(openApiSpecification, "$.info.description");
        assertThat(description).isEqualTo(SwaggerConfig.DESCRIPTION);

        var userPath = JsonPath.read(openApiSpecification, String.format("$.paths.%s", "/users/{id}"));
        assertThat(userPath).isNotNull();
    }
}
