package demo.app.interfaces;

import demo.app.IntegrationTest;
import demo.app.interfaces.DemoRestController.UserDto;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;

import static demo.app.interfaces.DemoRestController.DEMO_HEADER;
import static demo.app.interfaces.DemoRestController.DEMO_HEADER_VALUE;
import static org.assertj.core.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@IntegrationTest
class DemoRestControllerTest {

    private static final String USERS_RESOURCE = "/users/";

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private ObjectMapper objectMapper;

    @Test
    void givenUserId_getUserById_userIsReturned() throws Exception {
        final var userId = "abc";

        MvcResult result = mockMvc.perform(
                        get(USERS_RESOURCE + userId)
                                .accept(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andDo(print())
                .andReturn();

        var demoHeader = result.getResponse().getHeader(DEMO_HEADER);
        assertThat(demoHeader).isEqualTo(DEMO_HEADER_VALUE);

        UserDto user = objectMapper.readValue(result.getResponse().getContentAsString(), UserDto.class);
        assertThat(user).isNotNull();
        assertThat(user.id()).isEqualTo(userId);
    }
}